import java.io.File
import java.util.*

class Invoice {
    /**
     * The products that belong to the invoice.
     */
    var products = ArrayList<Product>()

    /**
     * Total price of all products.
     */
    val totalPrice: Float
        get() {
            return products.map { it.totalPrice }.sum()
        }

    companion object {
        /**
         * Creates an invoice from a file.
         */
        fun fromFile(fileName: String): Invoice {
            val invoice = Invoice()
            var columns: List<String> = emptyList()
            var processedHeader = false

            File(fileName).readLines().forEach { line ->
                // Don't process table dividers
                if (isDivider(line) || line.isBlank()) {
                    return@forEach
                }

                if (!processedHeader) {
                    columns = parseLine(line)
                    processedHeader = true
                    return@forEach
                }

                val product = Product()
                val lineItems = parseLine(line)
                columns.forEachIndexed { index, column ->
                    val value = lineItems[index]
                    when (column) {
                        "Item" -> product.name = value
                        "Unit Price" -> product.price = value.toFloat()
                        "Quantity" -> product.quantity = value.toFloat().toInt()
                        // "More information about a product is possible, e.g. description, but can be ignored."
                        "Description", "Pos", "Amount" -> {}
                        else -> throw ParseException("Unknown column: $column")
                    }
                }

                invoice.products.add(product)
            }

            return invoice
        }

        private fun parseLine(line: String): List<String> {
            return line.trim().trim('|').split('|').map { it.trim() }
        }

        /**
         * Checks if the given line is a divider in the table.
         */
        private fun isDivider(line: String): Boolean {
            return line.trim().startsWith("|*")
        }
    }
}